<?php
/*
 * Copyright(C) 2004, 2005 op5 AB
 * All rights reserved.
 *
 */


/*****************************************************************************
 *                          Code section start                               *
 *****************************************************************************/

// show an error if scan failed
function masscan_error($scan, $addr) {
	if(isset($scan["ip_resolve_status"]) && $scan["ip_resolve_status"] === false) {
		echo "<div class='info-notice info-notice-warning'>";
		echo "<h3>Failed to resolve <strong>". htmlspecialchars($addr)."</strong></h3>";
		echo "<p>You can still add the host, but it's very likely services will fail </p>";
		echo "</div>";
	} elseif (isset($scan["masscan_error"]) && $scan["masscan_error"] === true) {
		echo "<div class='info-notice info-notice-warning'>";
		echo "<h3>Could not reach <strong>". htmlspecialchars($addr)."</strong></h3>";
		echo "<p>You can still add the host, but it's very likely services will fail </p>";
		echo "</div>";
	}
}
function scan_error($target_arg){
	global $SUPPORT;
	if (is_array($target_arg)) {
		$target_arg = implode(" ", $target_arg);
	}
	echo "<img src=\"images/icons/shield-warning_24x24.png\" class=\"shield_warning\"/>
	<h1>Network Mapping Failed!</h1>
	<p>This usually happens beacuse of invalid target host specification (bad IP range). <br />
	Check your IP range ".($target_arg ? "(".$target_arg.")" : '')." and try again.<br /><br />
	Take note of any errors displayed.<br />
	Errors might also be logged to nacoma log file.<br />
	Should the problem persist, please contact $SUPPORT.</p>\n";
	html_end();
}

function parse_merlin_nodes($out) {
	// As soon as we have a number of pollers that's not completely insignificant,
	// they're almost certainly going to be arranged into groups of pollers.
	// Thus, caching the result for hostgroups we've already checked makes sense.
	$out_cache = array();
	$nodes = array('This server');
	$lines = explode("\n#", $out);
	foreach ($lines as $line) {
		if (!preg_match('/NAME=([^\n]+)/', $line, $matches)) {
			// first entry is empty, random stuff could happen
			continue;
		}
		$name = $matches[1];
		preg_match('/TYPE=([^\n]+)/', $line, $matches);
		$type = $matches[1];
		switch ($type) {
			case 'poller':
				preg_match('/HOSTGROUP=([^\n]+)/', $line, $matches);
				$hostgroup = $matches[1];
				if (isset($out_cache[$hostgroup])) {
					if ($out_cache[$hostgroup])
						$nodes[] = $name;
					break;
				}
				$hg_obj = new hostgroup_nacoma_Model($hostgroup);
				if ($hg_obj->can_user_read($hg_obj->id)) {
					$nodes[] = $name;
					$out_cache[$hostgroup] = true;
				}
				else {
					$out_cache[$hostgroup] = false;
				}
				break;
			case 'peer':
			case 'master':
				$nodes[] = $name;
				break;
			default:
				// not a peer, poller or master? lame, let's skip it
				break;
		}
	}
	return $nodes;
}

// Return a array of merlin nodes thats active
function get_merlin_nodes() {
	$sysret = probe("GET_MERLIN_NODES");
	return parse_merlin_nodes($sysret['buf']);
}


/**
 * Expands a numerical range on "," and "-"
 * Example: 1,3-5 => [1,3,4,5]
 * @return array[]
 */
function expand_numerical_range($ip){
	$split_range = explode(',', $ip);
	$ret_array = array();
	foreach ($split_range as $value) {
		$ret_array = array_merge($ret_array, range_split($value));
	}
	return $ret_array;
}

/**
 * Combines the values of $arr1 with the values of $arr2
 * joined by a "."
 * In other words: the cartesian product of $arr1 and $arr2
 * @return array[]
 */
function rebuild_ip_range(array $arr1, array $arr2){
	$combinations = array();

	foreach ($arr1 as $val1) {
		foreach ($arr2 as $val2) {
			array_push($combinations, $val1 .".". $val2);
		}
	}

	return $combinations;
}

/**
 * Expands a range that is specified with a dash "-"
 * into all separate numbers.
 * Example:1-3 => [1,2,3]
 * @return array[]
 */
function range_split($range_with_dash)
{
	$ret_array = array();
	if (strpos($range_with_dash, "-") === false) {
		array_push($ret_array, $range_with_dash);
		return $ret_array;
	}
	$split_range = explode("-", $range_with_dash);

	if ($split_range[0] === "") {
		$split_range[0] = 0;
	}
	if ($split_range[1] === "") {
		$split_range[1] = 255;
	}

	for ($i=$split_range[0]; $i <= $split_range[1]; $i++) {
		array_push($ret_array, strval($i));
	}

	if (strpos($split_range[1], "/")) {
		$subnet = explode("/", $split_range[1]);
		array_walk(
			$ret_array,
			function (&$value, $key, $subnet){
				$value=$value."/".$subnet[1];
			},
			$subnet
		);
	}

	return $ret_array;
}

/**
 * Parses an IP range to a list of IP
 * @param string $ip_range
 * @return array[IPs]
 * Example:
 * 127.0,1.0.1-2 =>
 * [127.0.0.1,
 * 	127.0.0.2,
 * 	127.1.0.1,
 * 	127.1.0.2]
 */
function parse_ip_range($ip_range){
	$parts = explode('.', $ip_range);
	$expanded_parts = array();
	foreach ($parts as $part) {
		array_push($expanded_parts, expand_numerical_range($part));
	}
	if (!is_array($parts) || count($parts) != 4) {
		throw new Exception("Not an valid IP", 1);
	}
	return rebuild_ip_range(
		rebuild_ip_range(
			rebuild_ip_range(
				$expanded_parts[0],
				$expanded_parts[1]
			),
			$expanded_parts[2]
		),
		$expanded_parts[3]
	);
}

function probe_wrapper($module, $arr, $target_arg){
	$sysret = probe($module, $arr);

	if($sysret['value'] > 1) {
		op5log::instance('nacoma')->log(
			'error',
			"Ping sweep failed with exit code {$sysret['value']}. Error: {$sysret['buf']}");
		scan_error($target_arg);
	}

	$hosts = explode("\n", $sysret['buf']);
	$ret_arr = array();
	foreach ($hosts as $host) {
		$pattern = "/(\S+) \(( ?\S+)\)/m";
		if (preg_match($pattern, $host, $matches)) {
			$ret_arr[$matches[2]] = $matches[1];
		}
	}
	return $ret_arr;
}


// run a pingsweep over a selected list of target ranges. Report only UP
function ping_sweep($target_arg) {
	$ip_range = explode(' ', $target_arg);
	$ip_range_list = array();
	$ip_list = array();
	$ip_cidr_list = array();

	foreach ($ip_range as $addr) {
		try {
			$ip_range_list = array_merge($ip_range_list, parse_ip_range($addr));
		} catch (\Throwable $e) {
			scan_error($addr);
			return;
		}
	}

	foreach ($ip_range_list as $value) {
		$cidr_regex = "/^([0-9]{1,3}\.){3}[0-9]{1,3}(\/([0-9]|[1-2][0-9]|3[0-2]))$/m";
		if (preg_match($cidr_regex, $value)) {
			array_push($ip_cidr_list, $value);
		} else {
			array_push($ip_list, $value);
		}
	}

	$ret_list = array();
	if (!empty($ip_list)) {
		$ret_list = array_merge($ret_list, probe_wrapper("PING_SWEEP", $ip_list, $target_arg));
	}
	if (!empty($ip_cidr_list)) {
		foreach ($ip_cidr_list as $value) {
			$ret_list = array_merge($ret_list, probe_wrapper("PING_SWEEP_CIDR", $value, $target_arg));
		}
	}

	return($ret_list);
}

/* makes a pretty-list of checkboxes for services to be selected */
function make_checklist_checkboxes($service, $inp_name) {
	echo "<tr><td colspan=\"2\"><input type=\"checkbox\" op5_xtra=\"".$inp_name."\" id=\"{$inp_name}[all]\" class=\"check_all\"> &nbsp;<label for=\"{$inp_name}[all]\">Select all</label></td></tr>\n";
	foreach($service as $srv_base => $srv_array) {
		echo "  <tr class='header'><td colspan=\"2\">\n";
		echo "    " . strtoupper($srv_base) . "\n";
		echo "  </td></tr>\n";
		foreach($srv_array as $srv_name => $srv_desc) {
			echo "      <tr>\n";

			echo "        <td style=\"width: 10px;\">\n";
			echo "           <input type='checkbox' id='{$inp_name}[$srv_name]' name='{$inp_name}[$srv_name]'";
			// only PING is checked by default
			if($srv_desc === 'PING') echo " checked='checked'";
			echo ">\n      </td>\n";
			echo "			<td style=\"border-right: 1px solid #cdcddc\"><label for='{$inp_name}[$srv_name]'>$srv_desc</label></td>\n";
			echo "</tr>\n";
		}
	}
}

function get_chkcommands() {
	global $DATADIR;
	$chkcommands_file = 'chkcommands';
	$retval = 0;
	$output = array();

	$buf = read_file($DATADIR . $chkcommands_file);
	$lines = explode("\n", $buf);
	$cmd = array();
	foreach($lines as $line) {
		if(!strlen($line)) continue;
		$str = preg_split("/[\t]+/", $line);
		if (count($str) != 3)
			continue;
		$str[0] = trim($str[0]);
		$cmd[$str[0]]['service_description'] = trim($str[2]);
		$command = explode('!', trim($str[1]), 2);
		$cmd[$str[0]]['check_command'] = $command[0];
		$cmd[$str[0]]['check_command_args'] = isset($command[1])?$command[1]:'';
	}
	return $cmd;
}

/* wrapper for exec_cmd. scans $target and returns any open ports,
 * ordered neatly in an associative array */
function port_scan($target) {
	// do the scan
	$cmdret = probe("PORT_SCAN", $target);
	if(isset($cmdret['ip_resolve_status']) && $cmdret['ip_resolve_status'] === false) {
		$ret['ip_resolve_status'] = $cmdret['ip_resolve_status'];
		op5log::instance('nacoma')->log(
			'error',
			"port scan failed with exit code {$cmdret['value']}. Error: {$cmdret['buf']}");
		$net[0] = 'ping';
		$ret['net'] = $net;
		return $ret;
	} elseif (!isset($cmdret['ip_resolve_status']) && $cmdret['value'] > 0) {
		$ret['masscan_error'] = true;
		op5log::instance('nacoma')->log(
			'error',
			"port scan failed with exit code {$cmdret['value']}. Error: {$cmdret['buf']}");
		$net[0] = 'ping';
		$ret['net'] = $net;
		return $ret;
	}

	$net[0] = 'ping';

	#Masscan does not return the name of the service on the default port
	#This mimics previous tools functionality
	$default_port_service = array(
		20 => "ftp-data",
		21 => "ftp",
		22 => "ssh",
		23 => "telnet",
		24 => "priv-mail",
		25 => "smtp",
		53 => "domain",
		80 => "http",
		110 => "pop3",
		119 => "nntp",
		135 => "msrpc",
		143 => "imap",
		220 => "imap3",
		443 => "https",
		993 => "imaps",
		1248 => "hermes",
		3306 => "mysql",
		5432 => "postgresql",
		5666 => "nrpe",
		6556 => "check_mk",
		9999 => "abyss"
	);

	$masscan_result = explode("\n", $cmdret['buf']);
	foreach ($masscan_result as $row) {
		$hostname = false;
		$pattern = "/(\d*)\/(tcp)/m";
		if (preg_match($pattern, $row, $matches)) {
			$net[(int)$matches[1]] = $default_port_service[$matches[1]];
		}
	}

	$ret['net'] = $net;
	return($ret);
}

function filter_monitored_hosts(&$target_list) {
	// remove those already configured
	$result = nacoma_db::exec_query('SELECT host_name, address FROM host ' .
		'WHERE host_name != \'\'');

	while($row = $result->fetch(PDO::FETCH_NUM)) {
		$ip = $row[1];
		$dns_name = $row[0];
		if(isset($target_list[$ip])) {
			unset($target_list[$ip]);
		}
		elseif(in_array($dns_name, $target_list)) {
			$tmp = array_search($dns_name, $target_list);
			unset($target_list[$tmp]);
		}
	}
	unset($result);
}

function do_snmp_scan($function, $args) {
	global $poller;
	if ($poller) {
		$out = probe('SNMP_SCAN', array('function' => $function, 'args' => $args));
		$out = preg_replace("/.*#--- REMOTE OUTPUT START ---\n*(.*)\n*#--- REMOTE OUTPUT DONE ---.*/ms", "$1", $out['buf']);
		return unserialize($out);
	}
	return call_user_func_array($function, $args);
}

function wmi_service_scan($address, $username, $password) {
	$ret = probe('WMI_SERVICE_SCAN', array('address' => $address, 'username' => $username, 'password' => $password));
	if ($ret['value'])
		return false;
	$ret = $ret['buf'];
	$ret = trim($ret);
	$svcs = explode("\n", $ret);
	array_shift($svcs);
	array_shift($svcs);
	$services = array();
	foreach ($svcs as $svc) {
		$svc = explode('|', $svc);
		$services[] = $svc[0];
	}
	return $services;
}

function nrpe_win_service_scan($address) {
	$max_left = 1000;
	$services = array();
	$options = array(
		'address' => $address,
		'arguments_allowed' => true,
		'start' => 0
	);
	$new_services = false;
	do {
		$ret = probe('WIN_SERVICE_SCAN', $options);
		if ($ret['value'] > 1 && $options['arguments_allowed']) {
			# The scan failed, but as NRPE args were used in the scan, that
			# could be why. Let's disable the use of args and give it another
			# shot. $new_services has still not been populated (still false),
			# so the while condition is still true at this point.
			$options['arguments_allowed'] = false;
			continue;
		}

		if (empty($ret['buf'])) {
			echo "No data returned from plugin\n";
			break;
		}

		$new_services = array();
		$result = explode(',', $ret['buf']);
		if (!empty($result)) {
			foreach ($result as $data) {
				$data = trim($data);
				if (trim($data === ''))
					continue;
				$new_services[$data] = trim($data);
			}
		}
		$new_keys = array_keys($new_services);
		$keys = array_keys($services);
		if (!empty($keys) && $keys[0] == $new_keys[0]) {
			# we are always gettings the same result back probably since
			# NRPE allows arguments but external script doesn't
			echo "<b>It seems that you need to allow arguments for external scripts in NSCLIENT.<br />" .
				"Either set all allowed_arguments=1 or remove them completely.<br />" .
				"As things are now - you will only get the first 20 services.</b>";
			break;
		}
		$services = array_merge($services, $new_services);
		$max_left -= count($new_services);
		$options['start'] += count($new_services);
	} while ($max_left > 0 && ($new_services === false || count($new_services) === 20));

	if (empty($services) && $max_left === 1000) {
		echo "No service list returned from host.\n";
		return false;
	}

	return $services;
}
